import threading

import requests
from bs4 import BeautifulSoup

base_url = 'https://hudoc.echr.coe.int/app/conversion/docx/html/body?library=ECHR&id='


def get_full_text_from_html(html_text):
    # This method turns the html code from the summary page into text
    # It has different cases depending on the first character of the CELEX ID
    # Should only be used for summaries extraction
    soup = BeautifulSoup(html_text, "html.parser")
    for script in soup(["script", "style"]):
        script.extract()  # rip it out
    text = soup.get_text()
    # break into lines and remove leading and trailing space on each
    lines = (line.strip() for line in text.splitlines())
    # break multi-headlines into a line each
    chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
    # drop blank lines
    text = '\n'.join(chunk for chunk in chunks if chunk)
    text = text.replace(",", "_")
    return text


def download_full_text_main(df, threads):
    item_ids = df['itemid']
    eclis = df['ecli']
    length = item_ids.size
    if length > threads:
        at_once_threads = int(length / threads)
    else:
        at_once_threads = length
    all_dict = list()
    threads = []
    for i in range(0, length, at_once_threads):
        curr_ids = item_ids[i:(i + at_once_threads)]
        curr_ecli = eclis[i:(i + at_once_threads)]
        t = threading.Thread(target=download_full_text_separate, args=(curr_ids, curr_ecli, all_dict))
        threads.append(t)
    for t in threads:
        t.start()
    for t in threads:
        t.join()

    json_file = list()
    for l in all_dict:
        if len(l) > 0:
            json_file.extend(l)
    return json_file


def download_full_text_separate(item_ids, eclis, dict_list):
    full_list = []
    eclis = eclis.reset_index(drop=True)
    item_ids = item_ids.reset_index(drop=True)

    def download_html(item_ids, eclis):
        retry_ids = []
        retry_eclis = []
        for i in range(len(item_ids)):
            item_id = item_ids[i]
            ecli = eclis[i]
            try:
                r = requests.get(base_url + item_id, timeout=1)
                json_dict = {
                    'item_id': item_id,
                    'ecli': ecli,
                    'full_text': get_full_text_from_html(r.text)
                }
                full_list.append(json_dict)
            except Exception:
                retry_ids.append(item_id)
                retry_eclis.append(ecli)
        return retry_ids, retry_eclis

    retry_ids, retry_eclis = download_html(item_ids, eclis)
    download_html(retry_ids, retry_eclis)
    dict_list.append(full_list)



import json
import logging
import os
from pathlib import Path

from echr_extractor.ECHR_html_downloader import download_full_text_main
from echr_extractor.ECHR_metadata_harvester import get_echr_metadata
from echr_extractor.ECHR_nodes_edges_list_transform import echr_nodes_edges

"""
I have replaced the function definition to take all arguments n eede to call read_echr_metadata and I have also
replaced the file naming lines. The old code is commented out I didn't delete anything. :)
On top of this the lines which deal with defining default values have been commented out because this is
handled in read_echr_metadata. It can also be done here but then it should be removed from the other method. 
It will probably be necissary to add some file handling to prevent overwriting. I'm not sure if you have plans for this
already but feel free to shoot me a message about it seeing as I did something similar for the ECHR branch. @Benjamin
"""


def get_echr(start_id=0, end_id=None, start_date=None, count=None, end_date=None, verbose=False, save_file='y',
             fields=None, link=None, language=None):
    if language is None:
        language = ["ENG"]
    if count:
        end_id = int(start_id) + count
    logging.info(f"--- STARTING ECHR DOWNLOAD FOR  ---")
    df = get_echr_metadata(start_id=start_id, end_id=end_id, start_date=start_date, end_date=end_date,
                           verbose=verbose, fields=fields, link=link, language=language)
    if df is False:
        return False
    if save_file == "y":
        filename = determine_filename(start_id, end_id, start_date, end_date)
        Path('data').mkdir(parents=True, exist_ok=True)
        file_path = os.path.join('data', filename + '.csv')
        df.to_csv(file_path, index=False)
        logging.info("\n--- DONE ---")
        return df
    else:
        logging.info("\n--- DONE ---")
        return df


def determine_filename(start_id, end_id, start_date, end_date):
    if end_id:
        if start_date and end_date:
            filename = f"echr_metadata_index_{start_id}-{end_id}_dates_{start_date}-{end_date}"
        elif start_date:
            filename = f"echr_metadata_{start_id}-{end_id}_dates_{start_date}-END"
        elif end_date:
            filename = f"echr_metadata_{start_id}-{end_id}_datesSTART-{end_date}"
        else:
            filename = f"echr_metadata_{start_id}-{end_id}_dates_START-END"
    else:
        if start_date and end_date:
            filename = f"echr_metadata_index_{start_id}-ALL_dates_{start_date}-{end_date}"
        elif start_date:
            filename = f"echr_metadata_{start_id}-ALL_dates_{start_date}-END"
        elif end_date:
            filename = f"echr_metadata_{start_id}-ALL_dates_START-{end_date}"
        else:
            filename = f"echr_metadata_{start_id}-ALL_dates_START-END"
    return filename


def get_echr_extra(start_id=0, end_id=None, start_date=None, count=None, end_date=None, verbose=False,
                   save_file='y', threads=10, fields=None, link=None, language=None):
    df = get_echr(start_id=start_id, end_id=end_id, start_date=start_date, end_date=end_date, verbose=verbose,
                  count=count, save_file='n', fields=fields, link=link, language=language)
    logging.info("Full-text download will now begin")
    if df is False:
        return False, False
    json_list = download_full_text_main(df, threads)
    logging.info("Full-text download finished")
    if save_file == "y":
        filename = determine_filename(start_id, end_id, start_date, end_date)
        filename_json = filename.replace("metadata", "full_text")
        Path('data').mkdir(parents=True, exist_ok=True)
        file_path = os.path.join('data', filename + '.csv')
        df.to_csv(file_path, index=False)
        file_path_json = os.path.join('data', filename_json + '.json')
        with open(file_path_json, "w") as f:
            json.dump(json_list, f)
        return df, json_list
    else:
        return df, json_list


def get_nodes_edges(metadata_path=None, df=None, save_file='y'):
    nodes, edges = echr_nodes_edges(metadata_path=metadata_path, data=df)
    if save_file == "y":
        Path('data').mkdir(parents=True, exist_ok=True)
        edges.to_csv(os.path.join('data', 'ECHR_edges.csv'), index=False, encoding='utf-8')
        nodes.to_csv(os.path.join('data', 'ECHR_nodes.csv'), index=False, encoding='utf-8')
        nodes.to_json(os.path.join('data', 'ECHR_nodes.json'), orient="records")
        edges.to_json(os.path.join('data', 'ECHR_edges.json'), orient="records")
        return nodes, edges

    return nodes, edges



import logging
import re

import dateparser
import numpy as np
import pandas as pd
from tqdm import tqdm

from echr_extractor.clean_ref import clean_pattern


def open_metadata(PATH_metadata):
    """
    Finds the ECHR metadata file and loads it into a dataframe
    
    param filename_metadata: string with path to metadata
    """
    try:
        df = pd.read_csv(PATH_metadata)  # change hard coded path
        return df
    except FileNotFoundError:
        logging.warning("File not found. Please check the path to the metadata file.")
        return False


def concat_metadata(df):
    agg_func = {'itemid': 'first', 'appno': 'first', 'article': 'first', 'conclusion': 'first', 'docname': 'first',
                'doctype': 'first',
                'doctypebranch': 'first', 'ecli': 'first', 'importance': 'first', 'judgementdate': 'first',
                'languageisocode': ', '.join, 'originatingbody': 'first',
                'violation': 'first', 'nonviolation': 'first', 'extractedappno': 'first', 'scl': 'first'}
    new_df = df.groupby('ecli').agg(agg_func)
    # print(new_df)
    return new_df


def get_language_from_metadata(df):
    df = concat_metadata(df)
    df.to_json('langisocode-nodes.json', orient="records")


def retrieve_edges_list(df):
    """
    Returns a dataframe consisting of 2 columns 'source' and 'target' which
    indicate a reference link between cases.

    params:
    df -- the node list extracted from the metadata
    df -- the complete dataframe from the metadata
    """
    edges = list()

    count = 0
    missing_cases = []
    bar = tqdm(total=len(df.index), colour="GREEN", position=0, leave=True)
    for index, item in df.iterrows():
        bar.update(1)
        eclis = []
        extracted_appnos = []
        if item.extractedappno is not np.nan:
            extracted_appnos = item.extractedappno.split(';')
        if item.scl is np.nan:
            continue
        """
            Split the references from the scl column i nto a list of references.

            Example:
            references in string: "Ali v. Switzerland, 5 August 1998, § 32, Reports of Judgments and 
            Decisions 1998-V;Sevgi Erdogan v. Turkey (striking out), no. 28492/95, 29 April 2003"

            ["Ali v. Switzerland, 5 August 1998, § 32, Reports of Judgments and 
            Decisions 1998-V", "Sevgi Erdogan v. Turkey (striking out), no. 
            28492/95, 29 April 2003"]
            """
        ref_list = item.scl.split(';')
        new_ref_list = [i.replace('\n', '') for i in ref_list]

        for ref in new_ref_list:
            app_number = re.findall("\d{3,5}/\d{2}", ref)

            app_number.extend(extracted_appnos)

            app_number = set(app_number)

            if len(app_number) > 0:
                # get dataframe with all possible cases by application number
                app_number = [';'.join(app_number)]
                case = lookup_app_number(app_number, df)
                if len(case) == 0:  # if failed try name?
                    case = lookup_casename(ref, df)
            else:  # if no application number in reference
                # get dataframe with all possible cases by casename
                case = lookup_casename(ref, df)

            components = ref.split(',')
            # get the year of case
            year_from_ref = get_year_from_ref(components)

            # remove cases in different language than reference
            case = remove_cases_based_on_language(case, components)
            case = remove_cases_based_on_year(case, year_from_ref)

            if len(case) > 0:
                for _, row in case.iterrows():
                    eclis.append(row.ecli)
            else:
                count = count + 1
                missing_cases.append(ref)

        eclis = set(eclis)
        eclis = [i for i in eclis if (i and i != item.ecli)]
        # add ecli to edges list
        if len(eclis) == 0:  # This should not have to happen at every iteration, concat might be slow
            continue
        for target in eclis:
            edges.append({'source': item.ecli, 'target': target})

    edges = pd.DataFrame.from_records(edges)
    return edges


def remove_cases_based_on_year(case, year_from_ref):
    for id, i in case.iterrows():
        if i.judgementdate is np.nan:
            continue
        try:
            date = dateparser.parse(i.judgementdate)
        except:
            date = False
        if date:
            year_from_case = date.year
            if year_from_case - year_from_ref == 0:
                case = case[
                    case['judgementdate'].str.contains(str(year_from_ref), regex=False,
                                                       flags=re.IGNORECASE, na=False)]
    return case


def remove_cases_based_on_language(cases, components):
    for id, it in cases.iterrows():
        if 'v.' in components[0]:
            lang = 'ENG'
        else:
            lang = 'FRE'

        if lang not in it.languageisocode:
            cases = cases[cases['languageisocode'].str.contains(lang, regex=False, flags=re.IGNORECASE, na=False)]
    return cases


def lookup_app_number(pattern, df):
    """
    Returns a list with rows containing the cases linked to the found app numbers.
    """
    row = df.loc[df['appno'].isin(pattern)]

    if row.empty:
        return pd.DataFrame()
    elif row.shape[0] > 1:
        return row
    else:
        return row


def lookup_casename(ref, df):
    """
    Process the reference for lookup in metadata.
    Returns the rows corresponding to the cases.

    - Example of the processing (2 variants) -

    Original reference from scl:
    - Hentrich v. France, 22 September 1994, § 42, Series A no. 296-A
    - Eur. Court H.R. James and Others judgment of 21 February 1986,
    Series A no. 98, p. 46, para. 81

    Split on ',' and take first item:
    Hentrich v. France
    Eur. Court H.R. James and Others judgment of 21 February 1986

    If certain pattern from CLEAN_REF in case name, then remove:
    Eur. Court H.R. James and Others judgment of 21 February 1986 -->
        James and Others

    Change name to upper case and add additional text to match metadata:
    Hentrich v. France --> CASE OF HENTRICH V. FRANCE
    James and Others --> CASE OF JAMES AND OTHERS
    """
    name = get_casename(ref)

    # DEV note: In case, add more patterns to clean_ref.py in future
    patterns = clean_pattern

    uptext = name.upper()

    if 'NO.' in uptext:
        uptext = uptext.replace('NO.', 'No.')

    if 'BV' in uptext:
        uptext = uptext.replace('BV', 'B.V.')

    if 'V.' in name:
        uptext = uptext.replace('V.', 'v.')
        lang = 'ENG'
    else:
        uptext = uptext.replace('C.', 'c.')
        lang = 'FRE'

    for pattern in patterns:
        uptext = re.sub(pattern, '', uptext)

    uptext = re.sub(r'\[.*', "", uptext)
    uptext = uptext.strip()
    row = df[df['docname'].str.contains(uptext, regex=False, flags=re.IGNORECASE)]

    # if len(row) == 0:
    #     print("no cases matched: ", name)

    return row


def get_casename(ref):
    count = 0
    if 'v.' in ref:
        slice_at_versus = ref.split('v.')  # skip if typo (count how many)
    elif 'c.' in ref:
        slice_at_versus = ref.split('c.')
    else:
        count = count + 1
        name = ref.split(',')
        return name[0]

    num_commas = slice_at_versus[0].count(',')

    if num_commas > 0:
        num_commas = num_commas + 1
        name = ",".join(ref.split(",", num_commas)[:num_commas])
    else:
        name = ref.split(',')
        return name[0]
    return name


def get_year_from_ref(ref):
    for component in ref:
        if '§' in component:
            continue
        component = re.sub('judgment of ', "", component)
        if dateparser.parse(component) is not None:
            date = dateparser.parse(component)
        elif ("ECHR" in component or "CEDH" in component):
            if ("ECHR" in component or "CEDH" in component):
                date = re.sub('ECHR ', '', component)
                date = re.sub('CEDH ', '', date)
                date = date.strip()
                date = re.sub('-.*', '', date)
                date = re.sub('\s.*', '', date)
                date = dateparser.parse(date)

    try:
        return date.year
    except:
        return 0


def echr_nodes_edges(metadata_path=None, data=None):
    """
    Create nodes and edges list for the ECHR data.
    """
    logging.info('\n--- COLLECTING METADATA ---\n')
    if metadata_path:
        data = open_metadata(metadata_path)
    elif data is None:
        logging.warning("No dataframe data provided. Returning...")
        return "", ""
    logging.info('\n--- EXTRACTING NODES LIST ---\n')
    # get_language_from_metadata(nodes)

    logging.info('\n--- EXTRACTING EDGES LIST ---\n')
    edges = retrieve_edges_list(data)

    # nodes.to_json(JSON_ECHR_NODES, orient="records")
    # edges.to_json(JSON_ECHR_EDGES, orient="records")
    return data, edges



import logging
import sys
from os.path import abspath

import echr_extractor

current_dir = (abspath(__file__))
correct_dir = '\\'.join(current_dir.replace('\\', '/').split('/')[:-2])
sys.path.append(correct_dir)
# print(sys.path)


if __name__ == '__main__':
    link = 'https://hudoc.echr.coe.int/eng#{%22fulltext%22:[%22(NOT%20\%22has%20been%20a%20violation%20of%20Article%206\%22)%20AND%20(\%22has%20been%20no%20violation%20of%20Article%206\%22)%22]}'
    logging.info('hey')


    df = echr_extractor.get_echr(link=link)
    b = 2



import logging
from datetime import datetime

import pandas as pd
import requests


def get_r(url, timeout, retry, verbose):
    """
    Get data from a URL. If this is uncuccessful it is attempted again up to a number of tries
    given by retry. If it is still unsuccessful the batch is skipped.
    :param str url: The data source URL.
    :param double timeout: The amount of time to wait for a response each attempt.
    :param int retry: The number of times to retry upon failure.
    :param bool verbose: Whether or not to print extra information.
    """
    count = 0
    max_attempts = 20
    while count < max_attempts:
        try:
            r = requests.get(url, timeout=timeout)
            return r
        except (requests.exceptions.ReadTimeout, requests.exceptions.ConnectTimeout):
            count += 1
            if verbose:
                logging.info(f"Timeout. Retry attempt {count}.")
            if count > retry:
                if verbose:
                    logging.info(f"Unable to connect to {url}. Skipping this batch.")
                return None
    return None


def basic_function(term, values):
    values = ['"' + i + '"' for i in values]
    main_body = list()
    cut_term = term.replace('"', '')
    for v in values:
        main_body.append(f"({cut_term}={v}) OR ({cut_term}:{v})")
    query = f"({' OR '.join(main_body)})"
    return query


def link_to_query(link):
    # Fixing brackets
    link = link.replace('%7B', '{')
    link = link.replace('%7D', '}')
    link = link.replace('%5B', '[')
    link = link.replace('%5D', ']')
    link = link.replace('%22', '"')
    link = link.replace('%27', "'")

    # fixing fulltext shenanigans - happen because of people using " in the queries.

    full_text_input = ''
    fulltext_end = -1
    fulltext_start = link.find('fulltext')
    if fulltext_start:
        start = link[fulltext_start:].find('[') + fulltext_start + 1
        fulltext_end = link[fulltext_start:].find(']') + fulltext_start
        fragment_to_fix = link[start:fulltext_end]
        full_text_input = '(' + "".join(fragment_to_fix.rsplit('"', 1)).replace('"', "", 1) + ')'.replace('\\','')
        full_text_input = full_text_input.replace("\\",'')
        b=2
        # removing first and last " elements and saving the output to put manually later
    if (fulltext_end):
        to_replace = link[fulltext_start - 1:fulltext_end + 1]
        link = link.replace(to_replace, '')

    extra_cases_map = {
        "bodyprocedure": '("PROCEDURE" ONEAR(n=1000) terms OR "PROCÉDURE" ONEAR(n=1000) terms)',
        "bodyfacts": '("THE FACTS" ONEAR(n=1000) terms OR "EN FAIT" ONEAR(n=1000) terms)',
        "bodycomplaints": '("COMPLAINTS" ONEAR(n=1000) terms OR "GRIEFS" ONEAR(n=1000) terms)',
        "bodylaw": '("THE LAW" ONEAR(n=1000) terms OR "EN DROIT" ONEAR(n=1000) terms)',
        "bodyreasons": '("FOR THESE REASONS" ONEAR(n=1000) terms OR "PAR CES MOTIFS" ONEAR(n=1000) terms)',
        "bodyseparateopinions": '(("SEPARATE OPINION" OR "SEPARATE OPINIONS") ONEAR(n=5000) terms OR "OPINION '
                                'SÉPARÉE" ONEAR(n=5000) terms)',
        "bodyappendix": '("APPENDIX" ONEAR(n=1000) terms OR "ANNEXE" ONEAR(n=1000) terms)'
    }

    def full_text_function(term, values):
        return f"({','.join(values)})"

    def date_function(term, values):
        values = ['"' + i + '"' for i in values]
        query = '(kpdate>=first_term AND kpdate<=second_term)'
        first = values[0]
        second = values[1]
        if first == '""':
            first = '"1900-01-01"'
        if second == '""':
            second = datetime.today().date()
        query = query.replace("first_term", first)
        query = query.replace("second_term", second)
        return query

    def advanced_function(term, values):
        body = extra_cases_map.get(term)
        query = body.replace("terms", ",".join(vals))
        return query

    query_map = {
        "docname": basic_function,
        "appno": basic_function,
        "scl": basic_function,
        "rulesofcourt": basic_function,
        "applicability": basic_function,
        "ecli": basic_function,
        "conclusion": basic_function,
        "resolutionnumber": basic_function,
        "separateopinions": basic_function,
        "externalsources": basic_function,
        "kpthesaurus": basic_function,
        "advopidentifier": basic_function,
        "documentcollectionid2": basic_function,
        "fulltext": full_text_function,
        "kpdate": date_function,
        "bodyprocedure": advanced_function,
        "bodyfacts": advanced_function,
        "bodycomplaints": advanced_function,
        "bodylaw": advanced_function,
        "bodyreasons": advanced_function,
        "bodyseparateopinions": advanced_function,
        "bodyappendix": advanced_function,
        "languageisocode": basic_function

    }
    start = link.index("{")
    link_dictionary = eval(link[start:])
    base_query = 'https://hudoc.echr.coe.int/app/query/results?query=contentsitename:ECHR' \
                 ' AND (NOT (doctype=PR OR doctype=HFCOMOLD OR doctype=HECOMOLD)) AND ' \
                 'inPutter&select={select}&sort=itemid%20Ascending&start={start}&length={length}'
    query_elements = list()
    if (full_text_input):
        query_elements.append(full_text_input)
    date_addition = ''
    for key in list(link_dictionary.keys()):
        if key == "kpdate":
            vals = link_dictionary.get(key)
            funct = query_map.get(key)
            date_addition = funct(key, vals)
        else:
            vals = link_dictionary.get(key)
            funct = query_map.get(key)
            query_elements.append(funct(key, vals))
    if date_addition:
        query_elements.append(date_addition)
    query_total = ' AND '.join(query_elements)
    final_query = base_query.replace('inPutter', query_total)

    return final_query


def get_echr_metadata(start_id, end_id, verbose, fields, start_date, end_date, link, language):
    """
    Read ECHR metadata into a Pandas DataFrame.
    :param int start_id: The index to start the search from.
    :param int end_id: The index to end search at, where the default fetches all results.
    :param date start_date: The point from which to save cases.
    :param date end_date: The point before which to save cases.
    :param bool verbose: Whether or not to print extra information.
    """
    data = []
    if not fields:
        fields = ['itemid', 'applicability', 'appno', 'article', 'conclusion', 'docname',
                  'doctype', 'doctypebranch', 'ecli', 'importance', 'judgementdate',
                  'languageisocode', 'originatingbody', 'violation', 'nonviolation',
                  'extractedappno', 'scl', 'publishedby', 'representedby', 'respondent',
                  'separateopinion', 'sharepointid', 'externalsources', 'issue', 'referencedate',
                  'rulesofcourt', 'DocId', 'WorkId', 'Rank', 'Author', 'Size', 'Path',
                  'Description', 'Write', 'CollapsingStatus', 'HighlightedSummary',
                  'HighlightedProperties', 'contentclass', 'PictureThumbnailURL',
                  'ServerRedirectedURL', 'ServerRedirectedEmbedURL', 'ServerRedirectedPreviewURL',
                  'FileExtension', 'ContentTypeId', 'ParentLink', 'ViewsLifeTime', 'ViewsRecent',
                  'SectionNames', 'SectionIndexes', 'SiteLogo', 'SiteDescription', 'deeplinks',
                  'SiteName', 'IsDocument', 'LastModifiedTime', 'FileType', 'IsContainer',
                  'WebTemplate', 'SecondaryFileExtension', 'docaclmeta', 'OriginalPath',
                  'EditorOWSUSER', 'DisplayAuthor', 'ResultTypeIdList', 'PartitionId', 'UrlZone',
                  'AAMEnabledManagedProperties', 'ResultTypeId', 'rendertemplateid']
    if link:
        META_URL = link_to_query(link)
        b = 2
    else:
        META_URL = 'http://hudoc.echr.coe.int/app/query/results' \
                   '?query=(contentsitename=ECHR) AND ' \
                   '(documentcollectionid2:"JUDGMENTS" OR ' \
                   'documentcollectionid2:"COMMUNICATEDCASES" OR ' \
                   'documentcollectionid2:"DECISIONS" OR ' \
                   'documentcollectionid2:"CLIN") AND ' \
                   'lang_inputter' \
                   '&select={select}' + \
                   '&sort=itemid Ascending' + \
                   '&start={start}&length={length}'

        # An example url: "https://hudoc.echr.coe.int/app/query/results?query=(contentsitename=ECHR)%20AND%20(documentcollectionid2:%22JUDGMENTS%22%20OR%20documentcollectionid2:%22COMMUNICATEDCASES%22%20OR%20documentcollectionid2:%22DECISIONS%22%20OR%20documentcollectionid2:%22CLIN%22)&select=itemid,applicability,application,appno,article,conclusion,decisiondate,docname,documentcollectionid,%20documentcollectionid2,doctype,doctypebranch,ecli,externalsources,extractedappno,importance,introductiondate,%20isplaceholder,issue,judgementdate,kpdate,kpdateAsText,kpthesaurus,languageisocode,meetingnumber,%20originatingbody,publishedby,Rank,referencedate,reportdate,representedby,resolutiondate,%20resolutionnumber,respondent,respondentOrderEng,rulesofcourt,separateopinion,scl,sharepointid,typedescription,%20nonviolation,violation&sort=itemid%20Ascending&start=0&length=200"

        if start_date and end_date:
            addition = f'(kpdate>="{start_date}" AND kpdate<="{end_date}")'
        elif start_date:
            end_date = datetime.today().date()
            addition = f'(kpdate>="{start_date}" AND kpdate<="{end_date}")'
        elif end_date:
            start_date = '1900-01-01'
            addition = f'(kpdate>="{start_date}" AND kpdate<="{end_date}")'
        else:
            addition = ''

        if addition:
            addition = " AND " + addition
            META_URL = META_URL.replace('&select', addition + '&select')

    META_URL = META_URL.replace(' ', '%20')
    META_URL = META_URL.replace('"', '%22')
    META_URL = META_URL.replace('%5C','')
    language_input = basic_function('languageisocode', language)
    if not link:
        META_URL = META_URL.replace('lang_inputter', language_input)

    META_URL = META_URL.replace('{select}', ','.join(fields))

    url = META_URL.format(start=0, length=1)
    logging.info(url)
    r = requests.get(url)
    resultcount = r.json()['resultcount']
    logging.info("available results: " + str(resultcount))

    if not end_id:
        end_id = resultcount
    if verbose:
        logging.info(f'Fetching {end_id - start_id} results from index {start_id} to index {end_id} ' +
                     f'{f" and filtering cases after {start_date}" if start_date else ""} {f"and filtering cases before {end_date}" if end_date else "."}')

    timeout = 60
    retry = 3
    if start_id + end_id > 500:  # HUDOC does not let you fetch more than 500 items in one go.
        for i in range(start_id, end_id, 500):
            if verbose:
                logging.info(" - Fetching information from cases {} to {}.".format(i, i + 500))
            # Format URL based on the incremented index.
            url = META_URL.format(start=i, length=500)
            if verbose:
                logging.info(url)

            # Get the response.
            r = get_r(url, timeout, retry, verbose)
            if r is not None:
                # Get the results list
                temp_dict = r.json()['results']
                # Get every document from the results list.
                for result in temp_dict:
                    data.append(result['columns'])

    else:
        # Format URL based on start and length
        url = META_URL.format(start=start_id, length=end_id)
        if verbose:
            logging.info(url)

        r = get_r(url, timeout, retry, verbose)
        if r is not None:
            # Get the results list
            temp_dict = r.json()['results']
            # Get every document from the results list.
            for result in temp_dict:
                data.append(result['columns'])

    if len(data) == 0:
        logging.info("Search results ended up empty")
        return False
    return pd.DataFrame.from_records(data)



# This file is required to create a python library

from pathlib import Path

from setuptools import find_packages, setup

p = Path("README.md")
long_descr = p.read_text()

setup(
    name='echr_extractor',
    packages=find_packages(include=['echr_extractor']),
    version='1.0.39',
    description='Library for extracting ECHR data',
    author='LawTech Lab',
    license='MIT',
    install_requires=["requests~=2.26.0", "pandas", "beautifulsoup4~=4.9.3", "dateparser", "tqdm"],
    author_email='a.gade@student.maastrichtuniversity.nl',
    keywords=['echr', 'extractor', 'european', 'convention', 'human', 'rights', 'european convention', 'human rights',
              'european convention on human rights'],
    long_description=long_descr,
    long_description_content_type='text/markdown',
    project_urls={
        "Bug Tracker": "https://github.com/maastrichtlawtech/extraction_libraries",
        "Build Source": "https://github.com/maastrichtlawtech/extraction_libraries",
    },
)
